/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: bitstring.c,v $
   Revision 1.2  2000/07/04 12:54:34  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:43  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

#define _POSIX_C_SOURCE 199506L

/* Local headers */

#include "pingutil.h"

/* Macros */

#define _P_bitstring_byte_offset(bit) \
  ((bit) >> 3)

#define _P_bitstring_byte_mask(bit) \
  (byte_t)(1 << ((bit) & 0x7))

#define _P_bitstring_getsize(nbits) \
  (size_t)((((nbits) - 1) >> 3) + 1)

/* External functions */

p_bitstring_t *P_bitstring_create(size_t nbits)
  {
  p_bitstring_t *bs;
  int len = _P_bitstring_getsize(nbits);

  if(len < 1)
    return(NULL);
  
  bs = P_new(p_bitstring_t);
  bs->length = len;
  bs->nbits = nbits;
  bs->bits = P_newa(len, byte_t);

  return(bs);
  }

/*
 */

bool_t P_bitstring_destroy(p_bitstring_t *bs)
  {
  if(! bs)
    return(FALSE);

  P_free(bs->bits);
  P_free(bs);

  return(TRUE);
  }

/*
 */

bool_t P_bitstring_compare(p_bitstring_t *bs1, p_bitstring_t *bs2)
  {
  int r = 0, i;
  byte_t *p, *q;

  if(bs1->nbits != bs2->nbits)
    return(FALSE);

  for(i = bs1->length, p = bs1->bits, q = bs2->bits; i--; i++, p++, q++)
    r |= ((*p & *q) != 0);

  return(r != 0);
  }

/*
 */

bool_t P_bitstring_clear(p_bitstring_t *bs, uint_t bit)
  {
  byte_t *b;

  if(! bs)
    return(FALSE);

  b = &(bs->bits[_P_bitstring_byte_offset(bit)]);

  *b &= ~(_P_bitstring_byte_mask(bit));

  return(TRUE);
  }

/*
 */

bool_t P_bitstring_set(p_bitstring_t *bs, uint_t bit)
  {
  byte_t *b;

  if(! bs)
    return(FALSE);

  b = &(bs->bits[_P_bitstring_byte_offset(bit)]);

  *b |= _P_bitstring_byte_mask(bit);

  return(TRUE);
  }

/*
 */

bool_t P_bitstring_clear_range(p_bitstring_t *bs, uint_t sbit, uint_t ebit)
  {
  uint_t sbyte, ebyte;

  if(!bs || (ebit < sbit))
    return(FALSE);

  sbyte = _P_bitstring_byte_offset(sbit);
  ebyte = _P_bitstring_byte_offset(ebit);

  if(sbyte == ebyte)
    {
    bs->bits[sbyte] &= ((0xFF >> (8 - (sbit & 0x07)))
                        | (0xFF << ((ebit & 0x07) + 1)));
    }
  else
    {
    bs->bits[sbyte] &= 0xFF >> (8 - (sbit & 0x07));

    while(++sbyte < ebyte)
      bs->bits[sbyte] = 0;

    bs->bits[ebyte] &= 0xFF << ((ebit & 0x07) + 1);
    }

  return(TRUE);
  }

/*
 */

bool_t P_bitstring_set_range(p_bitstring_t *bs, uint_t sbit, uint_t ebit)
  {
  uint_t sbyte, ebyte;
  
  if(!bs || (ebit < sbit))
    return(FALSE);

  sbyte = _P_bitstring_byte_offset(sbit);
  ebyte = _P_bitstring_byte_offset(ebit);

  if(sbyte == ebyte)
    {
    bs->bits[sbyte] |= ((0xFF << (sbit & 0x07))
                        & (0xFF >> (7 - (ebit & 0x07))));
    }
  else
    {
    bs->bits[sbyte] |= (0xFF << (sbit & 0x07));
    
    while(++sbyte < ebyte)
      bs->bits[sbyte] = 0xFF;
    
    bs->bits[ebyte] |= 0xFF >> (7 - (ebit & 0x07));
    }

  return(TRUE);
  }

/*
 */

bool_t P_bitstring_isset(p_bitstring_t *bs, uint_t bit)
  {
  uint_t b;
  
  if(! bs)
    return(FALSE);
  
  b = bs->bits[_P_bitstring_byte_offset(bit)];

  return((b & (_P_bitstring_byte_mask(bit))) != 0);
  }

/* end of source file */
