/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2001 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: dlobject.c,v $
   Revision 1.1  2000/12/31 03:56:59  markl
   New file.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <dlfcn.h>

/* Local headers */

#include "pingutil.h"

/* File scope variables */

/* Functions */

p_dlobject_t *P_dlobject_create(const char *path)
  {
  p_dlobject_t *obj;
  
  if(! path)
    return(FALSE);
  
  if(! *path)
    return(FALSE);
  
  obj = P_new(p_dlobject_t);
  obj->path = P_string_dup(path);
  obj->loaded = FALSE;

  return(obj);
  }

/*
 */

bool_t P_dlobject_load(p_dlobject_t *obj, bool_t lazy)
  {
  if(! obj)
    return(FALSE);

  obj->error = NULL;

  if(P_dlobject_isloaded(obj))
    return(FALSE);

  if(! (obj->handle = dlopen(obj->path, (lazy ? RTLD_LAZY : RTLD_NOW))))
    {
    obj->error = dlerror();
    return(FALSE);
    }
  
  obj->loaded = TRUE;

  return(TRUE);
  }

/*
 */

bool_t P_dlobject_unload(p_dlobject_t *obj)
  {
  if(! obj)
    return(FALSE);

  obj->error = NULL;
  
  if(! P_dlobject_isloaded(obj))
    return(FALSE);

  if(dlclose(obj->handle) != 0)
    {
    obj->error = dlerror();
    return(FALSE);
    }

  obj->loaded = FALSE;

  return(TRUE);
  }

/*
 */

bool_t P_dlobject_destroy(p_dlobject_t *obj)
  {
  if(! obj)
    return(FALSE);
  
  if(P_dlobject_isloaded(obj))
    return(FALSE);

  P_free(obj->path);
  P_free(obj);
  
  return(TRUE);
  }

/*
 */

void *P_dlobject_lookup(p_dlobject_t *obj, const char *symbol)
  {
  void *p;
  
  if(! obj || ! symbol)
    return(FALSE);

  obj->error = NULL;

  if(! P_dlobject_isloaded(obj))
    return(FALSE);

  if(! *symbol)
    return(FALSE);

  if(!(p = dlsym(obj->handle, symbol)))
    {
    obj->error = dlerror();
    return(NULL);
    }

  return(p);
  }

/* end of source file */
