/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: hex.c,v $
   Revision 1.4  2000/08/17 06:37:44  markl
   Changes to eliminate most compiler warnings.

   Revision 1.3  2000/07/22 11:43:08  markl
   Fixed parsing problem.

   Revision 1.2  2000/07/04 12:54:36  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:44  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <ctype.h>

/* Local headers */

#include "pingutil.h"

/* Functions */

char P_hex_tonibble(int v)
  {

  if(v < 0 || v > 15)
    return(NUL);

  return(v + ((v < 10) ? '0' : ('A' - 10)));
  }

/*
 */

int P_hex_fromnibble(char c)
  {

  if(!P_hex_isdigit(c))
    return(-1);

  c = toupper(c);
  
  return(c - ((c < 'A') ? '0' : ('A' - 10)));
  }

/*
 */

bool_t P_hex_tobyte(char *s, int v)
  {
  div_t d;

  if(v < 0 || v > 255 || !s) return(FALSE);

  d = div(v, 16);
  s[0] = P_hex_tonibble(d.quot);
  s[1] = P_hex_tonibble(d.rem);
  s[2] = NUL;

  return(TRUE);
  }

/*
 */

int P_hex_frombyte(char * const s)
  {
  int hi, lo;

  if(!s)
    return(-1);
  if((hi = P_hex_fromnibble(s[0])) < 0)
    return(-1);
  if((lo = P_hex_fromnibble(s[1])) < 0)
    return(-1);

  return(16 * hi + lo);
  }

/*
 */

bool_t P_hex_encode(char * const data, size_t len, char *s)
  {
  char *p, *q;
  int i;

  if(!data || !len || !s)
    return(FALSE);

  for(p = data, i = len, q = s; i--; p++, q += 2)
    P_hex_tobyte(q, (int)*p);
  *q = NUL;

  return(TRUE);
  }

/*
 */

bool_t P_hex_decode(char * const s, size_t len, char *data)
  {
  char *p, *q;
  int i;

  if(!s || !len || (len % 2) || !data)
    return(FALSE);

  for(q = s, i = len, p = data; i--; q += 2, p++)
    *p = (char)P_hex_frombyte(q);

  return(TRUE);
  }

/* end of source file */

