/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: io.c,v $
   Revision 1.4  2000/08/17 06:37:44  markl
   Changes to eliminate most compiler warnings.

   Revision 1.3  2000/07/21 08:14:29  markl
   Added new password input function.

   Revision 1.2  2000/07/04 12:54:36  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:45  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <termios.h>

/* Local headers */

#include "pingutil.h"

/* Macros */

#define P_IO_BLOCKSZ 40

/* Functions */

int P_io_getchar(uint_t delay)
  {
  struct termios t_old, t_new;
  int c;

  if(tcgetattr(STDIN_FILENO, &t_old))
    return(-1);
  t_new = t_old;

  t_new.c_iflag &= ~(IXON | IXOFF);
  t_new.c_lflag &= ~(ECHO | ICANON | ISIG | IEXTEN);
  t_new.c_cc[VMIN] = 0;
  t_new.c_cc[VTIME] = (delay * 10);

  if(tcsetattr(STDIN_FILENO, TCSANOW, &t_new))
    return(-1);
  if((c = getchar()) == EOF)
    clearerr(stdin);

  tcsetattr(STDIN_FILENO, TCSANOW, &t_old);

  return(c);
  }

/*
 */

int P_io_gets(FILE *fp, char *buf, size_t bufsz, char termin)
  {
  char *p = buf;
  int len = 0, ch;

  if(!buf || !bufsz)
    return(EOF);

  for(; (ch = fgetc(fp)); )
    {
    if((ch == EOF) || (ch == termin))
      break;
    *(p++) = ch;
    if(len++ == bufsz)
      break;
    }
  *p = NUL;
  if(!len && ch == EOF)
    return(EOF);

  return(len);
  }

/*
 */

int P_io_getpasswd(const char *prompt, char *buf, size_t bufsz)
  {
  struct termios t_old, t_new;
  int r;

  if(!isatty(STDIN_FILENO) || !isatty(STDOUT_FILENO))
    return(-1);

  tcgetattr(STDIN_FILENO, &t_old);
  t_new = t_old;

  t_new.c_lflag &= ~ECHO;

  if(tcsetattr(STDIN_FILENO, TCSANOW, &t_new) < 0)
    return(-1);
  
  fputs(prompt, stdout);
  fflush(stdout);

  r = P_io_gets(stdin, buf, bufsz, '\n');
  fputc('\n', stdout);

  if(tcsetattr(STDIN_FILENO, TCSANOW, &t_old) < 0)
    return(-1);

  return(r);
  }

/*
 */

char *P_io_getline(FILE *fp, char termin, int *len)
  {
  int c = 0, blk = 1, ch, l = 0;
  char *s, *p;

  if(!fp)
    return(NULL);

  s = P_newstr(P_IO_BLOCKSZ);

  for(p = s; (ch = fgetc(fp)); )
    {
    if((ch == EOF) || (ch == termin))
      break;
    *p = (char)ch, l++;
    if(++c == P_IO_BLOCKSZ)
      {
      c = 0;
      s = P_realloc(s, (P_IO_BLOCKSZ * ++blk), char);
      p = s + ((blk - 1) * P_IO_BLOCKSZ);
      }
    else p++;
    }

  if(!l && (ch == EOF))
    {
    P_free(s);
    return(NULL);
    }

  if(len)
    *len = l;
  *p = NUL;

  return(s = P_realloc(s, ++l, char));
  }

/* end of source file */
