/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: linklist.c,v $
   Revision 1.3  2000/08/17 06:37:44  markl
   Changes to eliminate most compiler warnings.

   Revision 1.2  2000/07/04 12:54:36  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:45  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* Local headers */

#include "pingutil.h"

/* Functions */

p_linklist_t *P_linklist_create(void)
  {
  p_linklist_t *l = P_new(p_linklist_t);

  l->head = l->tail = l->p = NULL;
  l->size = 0L;

  return(l);
  }

/*
 */

void P_linklist_destroy(p_linklist_t *l)
  {
  p_link_t *p, *q;

  if(!l)
    return;

  for(p = l->head; p; )
    {
    q = p;
    p = p->next;
    P_free(q);
    }
  P_free(l);
  }

/*
 */

bool_t P_linklist_store(p_linklist_t *l, const void *data)
  {
  p_link_t *p;

  if(!l || !data)
    return(FALSE);

  p = P_new(p_link_t);
  p->data = (void *)data;

  if(l->p == l->head) /* new head? */
    {
    p->next = l->head;
    p->prev = NULL;
    if(l->head)
      l->head->prev = p;
    l->head = p;
    if(!l->tail)
      l->tail = p;
    }
  else if(!l->p) /* new tail? */
    {
    p->next = NULL;
    p->prev = l->tail;
    if(l->tail)
      l->tail->next = p;
    l->tail = p;
    }
  else
    {
    p->next = l->p;
    p->prev = l->p->prev;
    if(l->p->prev)
      l->p->prev->next = p;
    l->p->prev = p;
    }
  l->p = p;
  l->size++;
  
  return(TRUE);
  }

/*
 */

void *P_linklist_restore(p_linklist_t *l)
  {
  
  if(!l)
    return(NULL);

  return(l->p ? l->p->data : NULL);
  }

/*
 */

bool_t P_linklist_search(p_linklist_t *l, const void *data)
  {
  void *d;

  for(P_linklist_move_head(l);
      (d = P_linklist_restore(l)) != NULL;
      P_linklist_move_next(l))
    {
    if(d == data)
      return(TRUE);
    }

  return(FALSE);
  }

/*
 */

bool_t P_linklist_push(p_linklist_t *l, const void *data)
  {
  
  if(!l || !data)
    return(FALSE);

  P_linklist_move_head(l);
  return(P_linklist_store(l, data));
  }

/*
 */

void *P_linklist_pop(p_linklist_t *l)
  {
  void *r;

  if(!l)
    return(NULL);
  if(!l->size)
    return(NULL);

  P_linklist_move_head(l);
  r = P_linklist_restore(l);
  P_linklist_delete(l);

  return(r);
  }

/*
 */

void *P_linklist_peek(p_linklist_t *l)
  {

  if(!l)
    return(NULL);
  if(!l->size)
    return(NULL);

  P_linklist_move_head(l);
  return(P_linklist_restore(l));
  }

/*
 */

bool_t P_linklist_enqueue(p_linklist_t *l, const void *data)
  {

  if(!l || !data)
    return(FALSE);

  P_linklist_move_tail(l);
  return(P_linklist_store(l, data));
  }

/*
 */

bool_t P_linklist_delete(p_linklist_t *l)
  {
  p_link_t *r;

  if(!l)
    return(FALSE);
  if(!l->p)
    return(FALSE);

  r = l->p;
  if(l->head == r)
    l->head = r->next;
  if(l->tail == r)
    l->tail = r->prev;
  if(l->p->prev)
    l->p->prev->next = l->p->next;
  if(l->p->next)
    l->p->next->prev = l->p->prev;
  l->p = l->p->next;
  l->size--;

  P_free(r);

  return(TRUE);
  }

/*
 */

bool_t P_linklist_move(p_linklist_t *l, int where)
  {

  if(!l)
    return(FALSE);

  switch(where)
    {
    case P_LINKLIST_HEAD:
      l->p = l->head;
      return(TRUE);

    case P_LINKLIST_TAIL:
      l->p = l->tail;
      return(TRUE);

    case P_LINKLIST_NEXT:
      if(l->p)
	{
	l->p = l->p->next;
	return(TRUE);
	}
      break;

    case P_LINKLIST_PREV:
      if(l->p)
        if(l->p->prev)
	  {
	  l->p = l->p->prev;
	  return(TRUE);
	  }
      break;
    }

  return(FALSE);
  }

/* end of source file */

  
