/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: memory.c,v $
   Revision 1.4  2000/07/04 12:54:36  markl
   More fixes.

   Revision 1.3  2000/06/29 21:51:32  markl
   Removed debug statements.

   Revision 1.2  2000/06/29 18:17:22  markl
   debug

   Revision 1.1  2000/06/14 06:02:45  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* Local headers */

#include "pingutil.h"

/* File scope variables */

static void (*__P_mem_errfunc)(void) = NULL;

/* Functions */

void *P_mem_manage(void *p, size_t n, uint_t ntries, uint_t delay,
		   bool_t clearf)
  {
  void *r;
  uint_t c = ntries;

  while(c--)
    {
    if((r = realloc(p, n)))
      {
      if(clearf)
        memset(r, 0, n);

      return(r);
      }
    else sleep(delay);
    }

  if(__P_mem_errfunc)
    __P_mem_errfunc();

  return(NULL);
  }

/*
 */

void P_mem_seterr(void (*func)(void))
  {

  __P_mem_errfunc = func;
  }

/*
 */

void *P_mem_free(void *p)
  {
  if(p)
    free(p);

  return(NULL);
  }

/*
 */

void P_mem_free_vec(char **v)
  {
  char **p;

  if(!v)
    return;

  for(p = v; *p; p++)
    P_free(*p);
  P_free(v);
  }

/*
 */

uint_t P_mem_va_free(uint_t n, ...)
  {
  int i, r = 0;
  va_list vp;
  void *p;

  va_start(vp, n);

  for(i = n; i--; )
    if((p = va_arg(vp, void *)))
      P_free(p), r++;

  va_end(vp);
  return(r);
  }

/*
 */

size_t P_mem_defrag(void *p, size_t elemsz, size_t len,
                    bool_t (*isempty)(void *elem))
  {
  uint_t mlen = 0, hlen = 0, i = 0;
  void *h, *m, *q;

  for(q = p; i < len; i++, q += elemsz)
    {
    if(isempty(q))
      {
      if(mlen)
	{
	memmove(h, m, (size_t)(mlen * elemsz));
	h += (mlen * elemsz);
	mlen = 0;
	}
      if(!(hlen++))
        h = q;
      }
    else
      {
      if(!hlen)
        continue;
      else if(!(mlen++))
        m = q;
      }
    }

  return((size_t)(len - hlen));
  }

/*
 */

p_buffer_t *P_buffer_create(size_t bufsz)
  {
  p_buffer_t *buf = P_new(p_buffer_t);
  buf->bufsz = bufsz;
  buf->buf = P_newb(bufsz);
  return(buf);
  }

/*
 */

p_buffer_t *P_buffer_resize(p_buffer_t *buf, size_t newsz)
  {
  if(newsz <= 0)
    return(NULL);

  buf->buf = P_realloc(buf->buf, newsz, char);
  buf->bufsz = newsz;

  return(buf);
  }

/*
 */

void P_buffer_destroy(p_buffer_t *buf)
  {
  if(!buf)
    return;

  P_free(buf->buf); /* ok if NULL */
  P_free(buf);
  }

/* end of source file */

