/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: misc.c,v $
   Revision 1.4  2000/08/17 07:23:30  markl
   Added new macro 'THREADED_LIBRARY' to correct some flaws in the
   configuration logic.

   Revision 1.3  2000/07/04 12:54:37  markl
   More fixes.

   Revision 1.2  2000/06/30 09:31:32  markl
   *** empty log message ***

   Revision 1.1  2000/06/14 06:02:45  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#ifdef THREADED_LIBRARY
#include <pthread.h>
#endif /* THREADED_LIBRARY */

/* Local headers */

#include "pingutil.h"

/* File scope variables */

#ifdef THREADED_LIBRARY

static pthread_once_t __P_misc_once = PTHREAD_ONCE_INIT;
static pthread_key_t __P_misc_key;

#endif /* THREADED_LIBRARY */

#ifdef THREADED_LIBRARY

/* File scope functions */

static void __P_misc_destructor(void *arg)
  {
  P_free(arg);
  }

/*
 */

static void __P_misc_init_once(void)
  {
  uint_t *seed = P_new(uint_t);

  *seed = time(NULL);
  
  pthread_key_create(&__P_misc_key, __P_misc_destructor);
  pthread_setspecific(__P_misc_key, (void *)seed);
  }

/* External functions */

uint_t P_misc_random(uint_t range)
  {
  uint_t *seed, r;

  pthread_once(&__P_misc_once, __P_misc_init_once);

  seed = (uint_t *)pthread_getspecific(__P_misc_key);
  r = (uint_t)rand_r(seed);

  return(range * r / RAND_MAX);
  }

#endif /* THREADED_LIBRARY */

/* end of source file */

