/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: pingutil.h.in,v $
   Revision 1.11  2001/01/21 08:50:44  markl
   Checks and conditional typedefs for in_addr_t and in_port_t

   Revision 1.10  2000/12/31 03:57:40  markl
   New definitions.

   Revision 1.9  2000/12/29 20:04:59  markl
   Updated exec() functions to take cwd argument, and added backward-compatible
   macros.

   Revision 1.8  2000/12/08 20:03:00  markl
   Bug fixes, and added new hashtable function.

   Revision 1.7  2000/08/13 06:02:01  markl
   #ifdef'ed out all of the scheduler functions for Linux, since there is no
   POSIX.4 timer support on Linux.

   Revision 1.6  2000/08/13 05:34:34  markl
   First cut at Linux port.

   Revision 1.5  2000/07/24 10:30:03  markl
   Typo fix.

   Revision 1.4  2000/07/22 11:38:11  markl
   Added new functions: strings, semaphores, and shared memory.

   Revision 1.3  2000/07/21 09:55:40  markl
   *** empty log message ***

   Revision 1.2  2000/07/21 08:13:55  markl
   Added defs for new functions.

   Revision 1.1  2000/07/13 00:35:08  markl
   *** empty log message ***

   Revision 1.6  2000/06/30 22:22:52  markl
   Added missing backslash.

   Revision 1.5  2000/06/30 09:28:20  markl
   Minor fixes.

   Revision 1.4  2000/06/15 23:37:55  markl
   Fixed prototype.

   Revision 1.3  2000/06/15 22:47:00  markl
   Added IDs to p_schedevt_t, added find function.

   Revision 1.2  2000/06/14 22:15:59  markl
   Added missing extern decl.

   Revision 1.1  2000/06/14 06:02:47  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

#ifndef __pingutil_h
#define __pingutil_h

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <stddef.h>
#include <time.h>
#include <sys/times.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <utime.h>
#include <stdarg.h>
#include <signal.h>
#include <semaphore.h>

#define SYNCHRONIZED

/* common definitions */

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif

#define TRUE (1)
#define FALSE (0)

#ifdef NUL
#undef NUL
#endif

#ifdef CRLF
#undef CRLF
#endif

#define NUL '\0'
#define CRLF "\r\n"

typedef char bool_t;

typedef unsigned char byte_t;

#if defined (linux) || defined (__hpux)
typedef unsigned int uint_t;
#endif

#define P_PING_VER "1.2 (POSIX/System V)"
#define P_PING_ID \
  "PING Utility Library (c) 1994-2000 PING Software Group"


#define P_random(N) \
  ((N) * rand() / RAND_MAX)

#define P_random_seed() \
  (srand((uint_t)time(NULL)))

#define P_offsetof(T, FIELD) \
  ((size_t)(((void *)(&(((T)NULL)->FIELD))) - ((void *)NULL)))

/* convenience macros */

#define P_max(A, B) \
  (((A) > (B)) ? (A) : (B))
#define P_min(A, B) \
  (((A) < (B)) ? (A) : (B))
#define P_sgn(A) \
  (((A) < 0) ? -1 : ((A) ? 1 : 0))

#define P_bit_set(I, B) \
  ((I) |= (1L << (B)))
#define P_bit_clr(I, B) \
  ((I) &= ~(1L << (B)))
#define P_bit_isset(I, B) \
  (((I) & (1L << (B))) ? TRUE : FALSE)

/* memory handling functions */

extern size_t P_mem_defrag(void *p, size_t elemsz, size_t len,
			   bool_t (*isempty)(void *elem));
extern void *P_mem_manage(void *p, size_t n, uint_t ntries, uint_t delay,
			  bool_t clearf);
extern void P_mem_seterr(void (*func)(void));
extern void *P_mem_free(void *p);
extern void P_mem_free_vec(char **p);
extern uint_t P_mem_va_free(uint_t n, ...);

#define P_MEM_MALLOCTRY 3
#define P_MEM_MALLOCDELAY 10

/* memory handling macros */

#define P_malloc(N, T) \
  (T *)P_mem_manage(NULL, (N) * sizeof(T), P_MEM_MALLOCTRY, \
		    P_MEM_MALLOCDELAY, FALSE)
#define P_calloc(N, T) \
  (T *)P_mem_manage(NULL, (N) * sizeof(T), P_MEM_MALLOCTRY, \
		    P_MEM_MALLOCDELAY, TRUE)
#define P_realloc(P, N, T) \
  (T *)P_mem_manage((void *)(P), (N) * sizeof(T), P_MEM_MALLOCTRY, \
		    P_MEM_MALLOCDELAY, FALSE)

#define P_zero(P, T) \
  memset((void *)P, 0, sizeof(T))
#define P_zeroa(P, N, T) \
  memset((void *)P, 0, (N) * sizeof(T))

#define P_free(P) \
  P_mem_free((void *)(P))
#define P_new(T) P_calloc(1, T)
#define P_newa(N, T) P_calloc((N), T)
#define P_newb(N) \
  P_mem_manage(NULL, (N), P_MEM_MALLOCTRY, P_MEM_MALLOCDELAY, TRUE)
#define P_newstr(N) P_calloc((N), char)
#define P_free_vec P_mem_free_vec
#define P_va_free P_mem_va_free

/* buffer functions */

typedef struct p_buffer_t
  {
  char *buf;
  size_t bufsz;
  void *hook;
  } p_buffer_t;

extern p_buffer_t *P_buffer_create(size_t bufsz);
extern p_buffer_t *P_buffer_resize(p_buffer_t *buf, size_t newsz);
extern void P_buffer_destroy(p_buffer_t *buf);

#define P_buffer_data(B) ((B)->buf)
#define P_buffer_size(B) ((B)->bufsz)

/* time related functions */

extern bool_t P_time_format(time_t t, char *s, const char *fmt, size_t fmtsz);
extern time_t P_time_scan(const char *s);

/* error handling functions */

extern void P_error_init(const char *progname);
extern void P_error_printf(const char *fmt, ...);
extern void P_error_print_usage(const char *usage);
extern void P_error_syserr(void);


extern int p_errno;
#define P_error_get_errno() (p_errno)
#define P_error_set_errno(E) p_errno = E

/* I/O functions */

extern int P_io_getchar(uint_t delay);
extern int P_io_gets(FILE *fp, char *buf, size_t bufsz, char termin);
extern int P_io_getpasswd(const char *prompt, char *buf, size_t bufsz);
extern char *P_io_getline(FILE *fp, char termin, int *len);

#define P_IO_GETCHARDELAY 10

/* I/O macros */

#define P_getchar() \
  P_io_getchar(P_IO_GETCHARDELAY)
#define P_gets(B, L) \
  P_io_gets(stdin, (B), (L), '\n')
#define P_getline(F, L) \
  P_io_getline((F), '\n', (L))

/* signals */

#define P_SIGNAL_MIN SIGHUP

#ifdef SIGXFSZ
#define P_SIGNAL_MAX SIGXFSZ
#else
#define P_SIGNAL_MAX 31
#endif

extern const char *P_signal_name(int sig);

/* shared memory */

typedef struct p_shmem_t
  {
  int fd;
  void *base;
  size_t size;
  char *name;
  } p_shmem_t;

extern p_shmem_t *P_shmem_create(const char *name, size_t size, mode_t mode);
extern void P_shmem_destroy(p_shmem_t *mem);

#define P_shmem_base(M) \
  (M)->base
#define P_shmem_size(M) \
  (M)->size
#define P_shmem_name(M) \
  (M)->name

/* binary semaphores */

typedef struct p_sem_t
  {
  char *name;
  sem_t *sem;
  } p_sem_t;

extern p_sem_t *P_sem_create(const char *name, mode_t mode);
extern void P_sem_destroy(p_sem_t *sem);

extern bool_t P_sem_wait(p_sem_t *sem);
extern bool_t P_sem_trywait(p_sem_t *sem);
extern bool_t P_sem_post(p_sem_t *sem);

#define P_sem_name(S) \
  (S)->name

/* string and parsing functions */

extern char *P_string_clean(char *s, char fillc);
extern char *P_string_tolower(char *s);
extern bool_t P_string_endswith(const char *s, const char *suffix);
extern bool_t P_string_isnumeric(const char *s);
extern char *P_string_strip(char *s);
extern uint_t P_string_wsplit(const char *s, char *buf, uint_t maxw,
                              uint_t maxc);
extern char **P_string_split(char *s, const char *sep, uint_t *len);
extern char *P_string_dup(const char *s);
extern char *P_string_chop(char *s, char termin);
extern char *P_string_chopx(char *s, char *termin);
extern char *P_string_duptermin(const char *s, char termin);
extern char **P_string_sortvec(char **v, size_t len);

extern size_t P_string_va_length(const char *s, ...);
extern char *P_string_va_copy(char *dest, ...);
extern char *P_string_va_ncopy(char *dest, size_t n, ...);
extern char **P_string_va_makevec(uint_t *len, ...);
extern char **P_string_valist2vec(const char *first, va_list vp, uint_t *slen);
extern uint_t P_string_hash(const char *s, uint_t modulo);
extern int P_string_compare(const void *s1, const void *s2);

/* bit strings */

typedef struct p_bitstring_t
  {
  byte_t *bits;
  size_t length;
  size_t nbits;
  } p_bitstring_t;

extern p_bitstring_t *P_bitstring_create(uint_t nbits);
extern bool_t P_bitstring_destroy(p_bitstring_t *bs);

extern bool_t P_bitstring_compare(p_bitstring_t *bs1, p_bitstring_t *bs2);
extern bool_t P_bitstring_clear(p_bitstring_t *bs, uint_t bit);
extern bool_t P_bitstring_set(p_bitstring_t *bs, uint_t bit);
extern bool_t P_bitstring_clear_range(p_bitstring_t *bs, uint_t sbit,
                                      uint_t ebit);
extern bool_t P_bitstring_set_range(p_bitstring_t *bs, uint_t sbit,
                                    uint_t ebit);
extern bool_t P_bitstring_isset(p_bitstring_t *bs, uint_t bit);

#define P_bitstring_isclear(BS, B) \
  (! P_bitstring_isset((BS), (B)))

#define P_bitstring_size(BS) \
  ((BS)->nbits)

#define P_bitstring_set_all(BS) \
  P_bitstring_set_range((BS), 0, ((BS)->nbits - 1))

#define P_bitstring_clear_all(BS) \
  P_bitstring_clear_range((BS), 0, ((BS)->nbits - 1))

/* vector management */

typedef struct p_vector_t
  {
  uint_t len;
  uint_t c;
  char **vs;
  char **v;
  uint_t blk;
  uint_t blksz;
  } p_vector_t;

extern p_vector_t *P_vector_start(uint_t resize_rate);
extern bool_t P_vector_store(p_vector_t *v, const char *s);
extern char **P_vector_end(p_vector_t *v, uint_t *len);
extern void P_vector_abort(p_vector_t *v);

#define P_vector_free P_free_vec

/* process maintenance functions */

extern int P_exec_run_cwd(char **argv, int fdin, int fdout, bool_t waitf,
                          const char *cwd);
extern int P_exec_va_run(int fdin, int fdout, bool_t waitf, ...);
extern int P_exec_runpty(char **argv, FILE **fpin, FILE **fpout);
extern int P_exec_pipefrom_cwd(char **argv, int *fd, const char *cwd);
extern int P_exec_pipeto_cwd(char **argv, int *fd, const char *cwd);
extern int P_exec_va_call(const char *arg, ...);
extern int P_exec_wait(pid_t pid);

#define P_exec_run(A, FI, FO, W) \
  P_exec_run_cwd(A, FI, FO, W, NULL)
#define P_exec_pipefrom(A, F) \
  P_exec_pipefrom_cwd(A, F, NULL)
#define P_exec_pipeto(A, F) \
  P_exec_pipeto_cwd(A, F, NULL)

/* file system functions */

struct p_dirlist
  {
  char **files;
  char **dirs;
  uint_t nfiles;
  uint_t ndirs;
  };

extern bool_t P_file_readdir(const char *path, struct p_dirlist *dir,
                             int flags);
extern bool_t P_file_traverse(const char *path,
                              bool_t (*examine)(const char *file,
                                                const struct stat *fst,
                                                uint_t depth, void *hook),
                              void *hook);
extern char *P_file_getcwd(void);

#define P_FILE_SHOWDOT 1
#define P_FILE_SHOW2DOT 2
#define P_FILE_SHOWHIDDEN 4
#define P_FILE_ADDSLASH 8
#define P_FILE_SEP 16
#define P_FILE_SORT 32

#define P_file_issymlink(S) \
  (((S)->mode & S_ISLNK) ? TRUE : FALSE)
#define P_file_touch(F) \
  (utime((F), NULL))

#define P_FILE_READ_LOCK 0
#define P_FILE_WRITE_LOCK 1

extern bool_t P_file_lock(FILE *fp, int type);
extern bool_t P_file_trylock(FILE *fp, int type);
extern bool_t P_file_unlock(FILE *fp, int type);

/* system info functions */

typedef struct p_sysinfo_t
  {
  char *login;
  char *fullname;
  char *homedir;
  char *shell;
  uid_t uid;
  gid_t gid;
  uid_t euid;
  gid_t egid;
  char *hostname;
  char *osname;
  char *osver;
  char *osrel;
  char *arch;
  pid_t pid;
  pid_t ppid;
  char *term;
  time_t stime;
  } p_sysinfo_t;

extern bool_t P_system_isingroup(const char *login, const char *group);
extern p_sysinfo_t *P_system_getinfo(void);
extern bool_t P_system_cdhome(void);

extern bool_t P_system_passwd_validate(const char *plaintext,
                                       const char *ciphertext);
extern bool_t P_system_passwd_generate(const char *plaintext, char *buf,
                                       size_t bufsz);


/* convenience functions */

extern uid_t P_system_get_uid(void);
extern gid_t P_system_get_gid(void);
extern pid_t P_system_get_pid(void);
extern char *P_system_get_login(void);
extern char *P_system_get_fullname(void);
extern char *P_system_get_homedir(void);
extern char *P_system_get_hostname(void);
extern char *P_system_get_term(void);

/* timer functions */

typedef struct p_timer_t
  {
  time_t created;
  struct tms t1;
  struct tms t2;
  clock_t usr;
  clock_t sys;
  float usr_time;
  float sys_time;
  } p_timer_t;

#define P_timer_user(T) \
  ((T)->user_time)

#define P_timer_system(T) \
  ((T)->system_time)

#define P_timer_created(T) \
  ((T)->created)

extern p_timer_t *P_timer_create(void);
extern void P_timer_destroy(p_timer_t *timer);

extern void P_timer_start(p_timer_t *timer);
extern void P_timer_resume(p_timer_t *timer);
extern void P_timer_stop(p_timer_t *timer);

/* hex utilities */

#define P_hex_isdigit(A) \
  ((((A) >= '0') && ((A) <= '9')) \
   || (((A) >= 'A') && ((A) <= 'F')) \
   || (((A) >= 'a') && ((A) <= 'f')))

extern char P_hex_tonibble(int v);
extern int P_hex_fromnibble(char c);

extern bool_t P_hex_tobyte(char *s, int v);
extern int P_hex_frombyte(char * const s);

extern bool_t P_hex_encode(char * const data, size_t len, char *s);
extern bool_t P_hex_decode(char * const s, size_t len, char *data);

/* dynamic linker functions */

typedef struct p_dlobject_t
  {
  void *handle;
  char *path;
  char *error;
  bool_t loaded;
  time_t last_access;
  void *hook;
  } p_dlobject_t;

extern p_dlobject_t *P_dlobject_create(const char *path);
extern bool_t P_dlobject_load(p_dlobject_t *obj, bool_t lazy);
extern bool_t P_dlobject_unload(p_dlobject_t *obj);
extern bool_t P_dlobject_destroy(p_dlobject_t *obj);
extern void *P_dlobject_lookup(p_dlobject_t *obj, const char *symbol);

#define P_dlobject_isloaded(OBJ) \
  ((OBJ)->loaded)
#define P_dlobject_error(OBJ) \
  ((OBJ)->error)
#define P_dlobject_path(OBJ) \
  ((OBJ)->path)

/* linked lists */

typedef struct p_link_t
  {
  void *data;
  struct p_link_t *next;
  struct p_link_t *prev;
  } p_link_t;

#define P_link_next(L) ((L)->next)
#define P_link_prev(L) ((L)->prev)
#define P_link_data(L) ((L)->data)

typedef struct p_linklist_t
  {
  p_link_t *head;
  p_link_t *tail;
  p_link_t *p;
  size_t size;
  } p_linklist_t;

#define P_linklist_head(L) ((L)->head)
#define P_linklist_tail(L) ((L)->tail)
#define P_linklist_size(L) ((L)->size)

extern p_linklist_t *P_linklist_create(void);
extern void P_linklist_destroy(p_linklist_t *l);
extern bool_t P_linklist_store(p_linklist_t *l, const void *data);
extern void *P_linklist_restore(p_linklist_t *l);

extern bool_t P_linklist_search(p_linklist_t *l, const void *data);

extern bool_t P_linklist_move(p_linklist_t *l, int where);
extern bool_t P_linklist_delete(p_linklist_t *l);

#define P_LINKLIST_HEAD 0
#define P_LINKLIST_TAIL 1
#define P_LINKLIST_NEXT 2
#define P_LINKLIST_PREV 3

#define P_linklist_move_head(L) P_linklist_move((L), P_LINKLIST_HEAD)
#define P_linklist_move_tail(L) P_linklist_move((L), P_LINKLIST_TAIL)
#define P_linklist_move_next(L) P_linklist_move((L), P_LINKLIST_NEXT)
#define P_linklist_move_prev(L) P_linklist_move((L), P_LINKLIST_PREV)
#define P_linklist_istail(L) ((L)->p ? FALSE : TRUE)
#define P_linklist_ishead(L) (((L)->p == (L)->head) ? TRUE : FALSE)
#define P_linklist_length(L) ((L)->size)

/* stacks */

typedef p_linklist_t p_stack_t;

#define P_stack_create() P_linklist_create()
#define P_stack_destroy(S) P_linklist_destroy(S)

extern bool_t P_linklist_push(p_linklist_t *l, const void *data);
extern void *P_linklist_pop(p_linklist_t *l);
extern void *P_linklist_peek(p_linklist_t *l);

#define P_stack_push(S, D) P_linklist_push((S), (D))
#define P_stack_pop(S) P_linklist_pop(S)
#define P_stack_peek(S) P_linklist_peek(S);

#define P_stack_depth(S) P_linklist_length(S)

/* queues */

typedef p_linklist_t p_queue_t;

#define P_queue_create() P_linklist_create()
#define P_queue_destroy(Q) P_linklist_destroy(Q)

extern bool_t P_linklist_enqueue(p_linklist_t *l, const void *data);

#define P_queue_enqueue(Q, D) P_linklist_enqueue((Q), (D))

#define P_queue_dequeue(Q) P_linklist_pop(Q)

#define P_queue_length(Q) P_linklist_length(Q)

/* hash tables */

typedef struct p_tag_t
  {
  char *key;
  void *data;
  } p_tag_t;

#define P_tag_key(T) ((T)->key)
#define P_tag_data(T) ((T)->data)

typedef struct p_hashtable_t
  {
  uint_t buckets;
  p_linklist_t **table;
  size_t size;
  } p_hashtable_t;

#define P_hashtable_size(H) ((H)->size)

extern p_hashtable_t *P_hashtable_create(uint_t buckets);
extern void P_hashtable_destroy(p_hashtable_t *h);
extern bool_t P_hashtable_set_hashfunc(uint_t (*func)(const char *s,
                                                      uint_t modulo));

extern bool_t P_hashtable_store(p_hashtable_t *h, const char *key,
                                const void *data);
extern void *P_hashtable_restore(p_hashtable_t *h, const char *key);
extern bool_t P_hashtable_delete(p_hashtable_t *h, const char *key);

extern char **P_hashtable_keys(p_hashtable_t *h, uint_t *len);

/* string heaps */

struct p_strheap_ent
  {
  uint_t offset;
  size_t len;
  char flags;
  };

typedef struct p_strheap_t
  {
  char *mem;
  struct p_strheap_ent *index;
  uint_t blk;
  uint_t iblk;
  uint_t bot;
  uint_t blocksz;
  size_t size;
  char flags;
  } p_strheap_t;

#define P_strheap_size(H) ((H)->size)

extern p_strheap_t *P_strheap_create(uint_t blocksz);
extern void P_strheap_destroy(p_strheap_t *h);

extern uint_t P_strheap_store(p_strheap_t *h, const char *s);
extern char *P_strheap_restore(p_strheap_t *h, uint_t index);
extern bool_t P_strheap_delete(p_strheap_t *h, uint_t index);

extern p_strheap_t *P_strheap_load(const char *path);
extern bool_t P_strheap_save(p_strheap_t *h, const char *path);

/* dynamic arrays */

typedef struct p_darray_t
  {
  void *mem;
  uint_t resize;
  size_t elemsz;
  uint_t size;
  uint_t isize;
  uint_t bot;
  uint_t del_count;
  char *free_list;
  uint_t iresize;
  } p_darray_t;

#define P_darray_size(A) ((A)->bot - (A)->del_count)
#define P_darray_last(A) ((A)->bot)

#define P_DARRAY_MAXRESIZE 100

extern p_darray_t *P_darray_create(uint_t resize, size_t elemsz);
extern void P_darray_destroy(p_darray_t *a);

extern void *P_darray_store(p_darray_t *a, const void *data, uint_t *index);
extern void *P_darray_restore(p_darray_t *a, uint_t index);
extern bool_t P_darray_delete(p_darray_t *a, uint_t index);

extern p_darray_t *P_darray_load(const char *path);
extern bool_t P_darray_save(p_darray_t *a, const char *path);
extern p_darray_t *P_darray_defragment(p_darray_t *a);
extern bool_t P_darray_iterate(p_darray_t *a,
                               bool_t (*iter)(void *elem, uint_t index,
                                              void *hook),
                               uint_t index, void *hook);

/* dynamic strings */

typedef struct p_dstring_t
  {
  char *mem;
  unsigned long p;
  unsigned long len;
  uint_t blk;
  uint_t blocksz;
  } p_dstring_t;

#define P_dstring_length(D) ((D)->len)

#define P_DSTRING_SEEK_ABS 0
#define P_DSTRING_SEEK_REL 1
#define P_DSTRING_SEEK_END 2

#define P_DSTRING_MINBLOCKSZ 80
#define P_DSTRING_LOADBLOCKSZ 4096

extern p_dstring_t *P_dstring_create(uint_t blocksz);
extern char *P_dstring_destroy(p_dstring_t *d);

extern bool_t P_dstring_putc(p_dstring_t *d, char c);
extern bool_t P_dstring_puts(p_dstring_t *d, const char *s);
extern char P_dstring_getc(p_dstring_t *d);
extern char *P_dstring_gets(p_dstring_t *d, char *s, size_t len, char termin);

extern bool_t P_dstring_seek(p_dstring_t *d, unsigned long where, int whence);
extern bool_t P_dstring_trunc(p_dstring_t *d, unsigned long size);
extern p_dstring_t *P_dstring_load(const char *path, uint_t blocksz);
extern bool_t P_dstring_save(p_dstring_t *d, const char *path);

#define P_dstring_backspace(D) P_dstring_seek((D), -1L, P_DSTRING_SEEK_REL)
#define P_dstring_rewind(D) P_dstring_seek((D), 0L, P_DSTRING_SEEK_ABS)
#define P_dstring_append(D) P_dstring_seek((D), 0L, P_DSTRING_SEEK_END)

/* real-time scheduler */

/* These functions are not available in Linux, since POSIX.4 timers are not
 * currently supported by Linux.
 */

#ifndef linux

#define P_SCHED_MASK_COUNT 5

typedef struct p_sched_t
  {
  timer_t timer;
  struct sigevent evp;
  void *hook;
  void (*handler)(void);
  p_linklist_t *events;
  } p_sched_t;

typedef struct p_schedevt_t
  {
  p_bitstring_t *mask[P_SCHED_MASK_COUNT];
  bool_t once;
  void (*handler)(struct p_schedevt_t *, time_t);
  void (*destructor)(struct p_schedevt_t *);
  void *hook;
  bool_t active;
  uint_t id;
  } p_schedevt_t;

extern bool_t P_sched_init(void);
extern bool_t P_sched_shutdown(void);

extern p_schedevt_t *P_sched_event_create(const char *timespec, bool_t once,
                                          void *hook,
                                          void (*handler)(p_schedevt_t *,
                                                          time_t),
                                          void (*destructor)(p_schedevt_t *),
                                          uint_t id);

extern bool_t P_sched_event_destroy(p_schedevt_t *e);

extern bool_t P_sched_event_activate(p_schedevt_t *e);
extern bool_t P_sched_event_deactivate(p_schedevt_t *e);
extern p_schedevt_t *P_sched_event_find(uint_t id);

#define P_sched_event_data(D) \
  (D)->hook

#define P_sched_event_id(D) \
  (D)->id

#endif /* ! linux */

#endif /* __pingutil_h */

/* end of library header */
