/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: pty.c,v $
   Revision 1.3  2000/08/17 06:37:44  markl
   Changes to eliminate most compiler warnings.

   Revision 1.2  2000/07/04 12:54:37  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:47  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <stropts.h>
#include <termios.h>
#include <stdlib.h>

/* Local headers */

#include "pingutil.h"
#include "pingnet.h"
#include "perrno.h"

/* Macros */

#define P_TERM_MASTER "/dev/ptmx"

/* Functions */

int P_pty_master(void)
  {
  int mfd;

  return(((mfd = open(P_TERM_MASTER, O_RDWR)) < 0) ? -1 : mfd);
  }

/*
 */

int P_pty_slave(int mfd)
  {
  char *s;
  int sfd;  
  bool_t f = FALSE;

  if(grantpt(mfd) >= 0)
    if(unlockpt(mfd) >= 0)
      if((s = (char *)ptsname(mfd)))
        f = TRUE;

  if(!f)
    {
    P_error_set_errno(P_EGETPTY);
    return(-1);
    }

  if((sfd = open(s, O_RDWR)) < 0)
    {
    P_error_set_errno(P_EOPEN);
    return(-1);
    }

  f = FALSE;
  if(ioctl(sfd, I_PUSH, "ptem") >= 0)
    if(ioctl(sfd, I_PUSH, "ldterm") >= 0)
      f = TRUE;

  if(!f)
    {
    P_error_set_errno(P_EIOCTL);
    return(-1);
    }

  return(sfd);
  }

/*
 */

int P_pty_exec(const char *path, char *const argv[], pid_t *p)
  {
  int mfd, sfd;
  pid_t pid;

  if(!isatty(STDIN_FILENO) || !isatty(STDOUT_FILENO))
    {
    P_error_set_errno(P_ENOTTY);
    return(-1);
    }

  if((mfd = P_pty_master()) < 0)
    {
    P_error_set_errno(P_EPTY);
    return(-1);
    }

  if(!P_tty_store(STDIN_FILENO))
    return(-1);

  switch(pid = fork())
    {
    case -1:
      P_error_set_errno(P_EFORK);
      return(-1);

    case 0: /* child */
      {
      setpgid(0, 0);

      if((sfd = P_pty_slave(mfd)) < 0) exit(-1);
      if(!P_tty_restore(sfd)) exit(-1);

      close(STDIN_FILENO);
      dup(sfd);
      close(STDOUT_FILENO);
      dup(sfd);
      close(STDERR_FILENO);
      dup(sfd);

      if(!P_tty_restore(sfd)) exit(-1);

      execv(path, argv);

      /* not reached */
      exit(-1);      
      }

    default: /* parent */
      if(p) *p = pid;
      return(mfd);
    }
  }

/* end of source file */






