/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: sem.c,v $
   Revision 1.4  2000/08/13 05:56:43  markl
   More Linux ports.

   Revision 1.3  2000/07/22 11:38:11  markl
   Added new functions: strings, semaphores, and shared memory.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <fcntl.h>
#include <sys/semaphore.h>

/* Local headers */

#include "pingutil.h"

/* Functions */

p_sem_t *P_sem_create(const char *name, mode_t mode)
  {
  p_sem_t *sem;
  sem_t *s;
  
#ifdef __hpux
#define SEMA_FAILURE  0       /* semaphore operation failed. */
  if((s = sem_open(name, (O_CREAT | O_RDWR), mode, 1)) == SEMA_FAILURE)
#else
  if((s = sem_open(name, (O_CREAT | O_RDWR), mode, 1)) == SEM_FAILED)
#endif
    return(NULL);
  
  sem = P_new(p_sem_t);
  sem->sem = s;
  sem->name = P_string_dup(name);

  return(sem);
  }

/*
 */

void P_sem_destroy(p_sem_t *sem)
  {
  if(!sem)
    return;

  sem_close(sem->sem);
  sem_unlink(sem->name);

  P_free(sem->name);
  P_free(sem);
  }

/*
 */

bool_t P_sem_wait(p_sem_t *sem)
  {
  if(!sem || !(sem->sem))
    return(FALSE);

  return(sem_wait(sem->sem) == 0);
  }

/*
 */

bool_t P_sem_trywait(p_sem_t *sem)
  {
  if(!sem || !(sem->sem))
    return(FALSE);

  return(sem_trywait(sem->sem) == 0);
  }

/*
 */

bool_t P_sem_post(p_sem_t *sem)
  {
  if(!sem || !(sem->sem))
    return(FALSE);

  return(sem_post(sem->sem) == 0);
  }

/* end of source file */
