/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: shmem.c,v $
   Revision 1.3  2000/08/17 06:37:44  markl
   Changes to eliminate most compiler warnings.

   Revision 1.2  2000/07/22 11:38:12  markl
   Added new functions: strings, semaphores, and shared memory.

   Revision 1.1  2000/07/21 09:55:41  markl
   *** empty log message ***

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

#define _POSIX_C_SOURCE 199309L

/* System headers */

#include <fcntl.h>
#include <sys/mman.h>

/* Local headers */

#include "pingutil.h"


/* Functions */

static size_t __P_shmem_round_size(size_t size)
  {
  long pagesize = sysconf(_SC_PAGESIZE);

  return((size_t)(size + pagesize - (size % pagesize)));
  }

/*
 */

p_shmem_t *P_shmem_create(const char *name, size_t size, mode_t mode)
  {
  p_shmem_t *m;
  int fd;
  bool_t init = TRUE;

  if((fd = shm_open(name, (O_CREAT | O_RDWR | O_EXCL), mode)) < 0)
    {
    if(errno == EEXIST)
      {
      init = FALSE;

      if((fd = shm_open(name, (O_CREAT | O_RDWR), mode)) < 0)
        return(NULL);
      }
    else
      return(NULL);
    }
  
  m = P_new(p_shmem_t);
  m->name = P_string_dup(name);
  m->fd = fd;
  m->size = __P_shmem_round_size(size);

  if(init)
    ftruncate(m->fd, (off_t)m->size);
  
  m->base = mmap(NULL, m->size, (PROT_READ | PROT_WRITE), MAP_SHARED, m->fd,
                 (off_t)0);


  if(init)
    memset(m->base, 0, m->size);

  return(m);
  }

/*
 */

void P_shmem_destroy(p_shmem_t *mem)
  {
  if(!mem)
    return;

  munmap(mem->base, mem->size);
  shm_unlink(mem->name);
  P_free(mem->name);
  P_free(mem);
  }

/* end of source file */
