/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: strings.c,v $
   Revision 1.6  2001/01/21 08:51:50  markl
   Minor bug fix.

   Revision 1.5  2000/12/13 11:28:36  markl
   Bug fixes.

   Revision 1.4  2000/08/17 06:37:45  markl
   Changes to eliminate most compiler warnings.

   Revision 1.3  2000/07/22 11:38:12  markl
   Added new functions: strings, semaphores, and shared memory.

   Revision 1.2  2000/07/04 12:54:39  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:48  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

#define _POSIX_C_SOURCE 199506L

/* System headers */

#include <ctype.h>

/* Local headers */

#include "pingutil.h"

/* Macros */

#define P_STRING_BLOCKSZ 40

/* Functions */

char *P_string_clean(char *s, char fillc)
  {
  char *p;

  if(!s)
    return(NULL);

  for(p = s; *p; p++)
    if(!isprint((int)*p) && !isspace((int)*p))
      *p = fillc;

  return(s);
  }

/*
 */

char *P_string_tolower(char *s)
  {
  char *p;

  if(!s)
    return(NULL);

  for(p = s; *p; p++)
    if(isupper((int)(*p)))
      *p = (char)tolower((int)(*p));

  return(s);
  }

/*
 */

uint_t P_string_wsplit(const char *s, char *buf, uint_t maxw, uint_t maxc)
  {
  uint_t w = 0;
  register uint_t c;

  if(!s || !buf || !maxc || !maxw)
    return(0);

  for(; w < maxw;)
    {
    for(; isspace((int)*s); s++);

    for(c = 0; isgraph((int)*s) && (c < (maxc - 1)); s++, c++, buf++)
      *buf = *s;
    if(!c)
      break;
    if(isgraph((int)*s))
      *(buf-1) = '\\';

    for(*buf = 0, buf += (maxc -c), w++; isgraph((int)*s); s++);
    }
  
  return(w);
  }

/*
 */

char **P_string_split(char *s, const char *sep, uint_t *len)
  {
  p_vector_t *vec;
  char *p;
  char *q = NULL;

  if(!s || !sep)
    return(NULL);
  if(!*s || !*sep)
    return(NULL);

  vec = P_vector_start(P_STRING_BLOCKSZ);
  if(!((p = strtok_r(s, sep, &q))))
    {
    P_vector_abort(vec);
    if(len)
      *len = NUL;
    return(NULL);
    }
  else P_vector_store(vec, P_string_dup(p));

  do
    {
    if((p = strtok_r(NULL, sep, &q)))
      P_vector_store(vec, P_string_dup(p));
    }
  while(p);

  return(P_vector_end(vec, len));
  }

/*
 */

char *P_string_dup(const char *s)
  {
  char *r;
  size_t len;

  if(!s)
    return(NULL);

  len = strlen(s) + 1;
  r = P_newstr(len);

  return((char *)memcpy((void *)r, (void *)s, len));
  }

/*
 */

char **P_string_sortvec(char **v, size_t len)
  {

  qsort((void *)v, len, sizeof(char *), P_string_compare);
  return(v);
  }

/*
 */

char *P_string_strip(char *s)
  {
  char *p, *q;
  size_t l, ll;

  if(!s)
    return(NULL);

  l = ll = strlen(s);

  for(p = s; *p && isspace((int)*p); p++, l--);

  if(! l)
    *s = NUL;
  else
    {
    for(q = s + (--ll); (l > 0) && isspace((int)*q); q--, l--);

    memmove((void *)s, (void *)p, (size_t)l);

    *(s + l) = NUL;
    }

  return(s);
  }

/*
 */

size_t P_string_va_length(const char *s, ... /* , NULL */)
  {
  va_list vp;
  size_t len = strlen(s);
  char *p;

  va_start(vp, s);
  while((p = va_arg(vp, char *)))
    len += strlen(p);
  va_end(vp);

  return(len);
  }

/*
 */

char *P_string_va_copy(char *dest, ... /* , NULL */)
  {
  va_list vp;
  char *cp = dest, *p;

  va_start(vp, dest);
  while((p = va_arg(vp, char *)))
    for(; *p; *(cp++) = *(p++));
  *cp = NUL;
  va_end(vp);

  return(dest);
  }

/*
 */

char *P_string_va_ncopy(char *dest, size_t n, ... /* , NULL */)
  {
  va_list vp;
  char *cp = dest, *p;
  uint_t i;

  va_start(vp, n);
  while((p = va_arg(vp, char *)))
    for(i = 0; (*p && (i++ < n)); *(cp++) = *(p++));
  *cp = NUL;
  va_end(vp);

  return(dest);
  }

/*
 */

char *P_string_chop(char *s, char termin)
  {
  char *p;

  if((p = strchr(s, termin)))
    *p = NUL;
  return(s);
  }

/*
 */

char *P_string_chopx(char *s, char *termin)
  {
  char *p;

  if((p = strpbrk(s, termin)))
    *p = NUL;
  return(s);
  }

/*
 */

char **P_string_valist2vec(const char *first, va_list vp, uint_t *len)
  {
  p_vector_t *vec;
  char *p;

  if(!vp)
    return(NULL);

  vec = P_vector_start(P_STRING_BLOCKSZ);
  if(first)
    P_vector_store(vec, first);

  while((p = va_arg(vp, char *)))
    P_vector_store(vec, p);

  return(P_vector_end(vec, len));
  }

/*
 */

char **P_string_va_makevec(uint_t *len, ... /* , NULL */)
  {
  va_list vp;
  char **v;

  va_start(vp, len);
  v = P_string_valist2vec(NULL, vp, len);
  va_end(vp);

  return(v);
  }

/*
 */

char *P_string_duptermin(const char *s, char termin)
  {
  char *r;
  size_t len;

  if(!s || !termin)
    return(NULL);

  len = strlen(s);
  r = P_newstr(len + 2);
  memcpy((void *)r, (void *)s, len);
  *(r + len) = termin;
  *(r + ++len) = NUL;

  return(r);
  }

/*
 */

bool_t P_string_endswith(const char *s, const char *suffix)
  {
  int len, slen;

  if(!s || !suffix)
    return(FALSE);

  slen = strlen(suffix);
  len = strlen(s);

  if(len >= slen)
    if(!strcmp(s + (len - slen), suffix))
      return(TRUE);

  return(FALSE);
  }

/* this is passed a void pointer to a character pointer (ugh) so that the
   sorting takes place by swapping pointers rather than by copying strings
*/

int P_string_compare(const void *s1, const void *s2)
  {
  char **p = (char **)s1, **q = (char **)s2;

  return(strcmp(*p, *q));
  }

/*
 */

uint_t P_string_hash(const char *s, uint_t modulo)
  {
  uint_t hashval = 0, i;
  char *p;

  if(!s || !modulo)
    return(0);
  if(!*s)
    return(0);

  for(p = (char *)s; *p; p++)
    {
    hashval = (hashval << 4) + *p;
    if((i = (hashval & 0xf0000000)))
      {
      hashval ^= (i >> 24);
      hashval ^= i;
      }
    }

  return(hashval % modulo);
  }

/*
 */

bool_t P_string_isnumeric(const char *s)
  {
  char *p;

  for(p = (char *)s; *p; p++)
    if(! isdigit((int)*p))
      return(FALSE);
  
  return(TRUE);
  }

/* end of source file */
