/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: tty.c,v $
   Revision 1.2  2000/07/04 12:54:39  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:49  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#include <termios.h>
#include <stdio.h>

/* Local headers */

#include "pingnet.h"
#include "perrno.h"

/* File scope variables */

static struct termios __P_tty_mode;
static struct termios __P_tty_buf;
static bool_t __P_tty_rawf = FALSE;
static bool_t __P_tty_storef = FALSE;

/* Functions */

bool_t P_tty_raw(int fd)
  {
  struct termios t;

  if(!isatty(fd))
    {
    P_error_set_errno(P_ENOTTY);
    return(FALSE);
    }

  if(tcgetattr(fd, &t))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  __P_tty_mode = t; /* save tty mode */

  t.c_iflag = 0;
  t.c_oflag &= ~OPOST;
  t.c_lflag &= ~(ECHO | ICANON | ISIG);
  t.c_cflag &= ~(CSIZE | PARENB);
  t.c_cflag |= CS8;
  t.c_cc[VMIN] = t.c_cc[VTIME] = 1;

  if(tcsetattr(fd, TCSANOW, &t))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  __P_tty_rawf = TRUE;

  return(TRUE);
  }

/*
 */

bool_t P_tty_unraw(int fd)
  {

  if(!isatty(fd))
    {
    P_error_set_errno(P_ENOTTY);
    return(FALSE);
    }

  if(!__P_tty_rawf)
    {
    P_error_set_errno(P_EINVAL);
    return(FALSE);
    }

  if(!tcsetattr(fd, TCSANOW, &(__P_tty_mode)))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  return(TRUE);
  }

/*
 */

bool_t P_tty_store(int fd)
  {

  if(!isatty(fd))
    {
    P_error_set_errno(P_ENOTTY);
    return(FALSE);
    }

  if(!tcgetattr(fd, &(__P_tty_buf)))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  __P_tty_storef = TRUE;
  return(TRUE);
  }

/*
 */

bool_t P_tty_restore(int fd)
  {

  if(!isatty(fd))
    {
    P_error_set_errno(P_ENOTTY);
    return(FALSE);
    }

  if(!__P_tty_storef)
    {
    P_error_set_errno(P_EINVAL);
    return(FALSE);
    }

  if(!tcsetattr(fd, TCSANOW, &(__P_tty_buf)))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  return(TRUE);
  }

/*
 */

bool_t P_tty_generic(int fd)
  {
  int i;

  if(!isatty(fd))
    {
    P_error_set_errno(P_ENOTTY);
    return(FALSE);
    }

  memset((void *)&__P_tty_buf, 0, sizeof(struct termios));

  __P_tty_buf.c_iflag = ICRNL | IXON | IXOFF;
  __P_tty_buf.c_oflag = OPOST;
  __P_tty_buf.c_cflag = CREAD | HUPCL;
  __P_tty_buf.c_lflag = ECHO | ECHOE | ECHOK | ICANON | ISIG | IEXTEN;

  for(i = 0; i < NCCS; i++)
    __P_tty_buf.c_cc[i] = -1;

  __P_tty_buf.c_cc[VEOF]   =	 3;
  __P_tty_buf.c_cc[VEOL]   =	28;
  __P_tty_buf.c_cc[VERASE] =	 8;
  __P_tty_buf.c_cc[VINTR]  =	21;
  __P_tty_buf.c_cc[VKILL]  =	 4;
  __P_tty_buf.c_cc[VQUIT]  =   255;
  __P_tty_buf.c_cc[VSUSP]  =   255;
  __P_tty_buf.c_cc[VSTART] =	17;
  __P_tty_buf.c_cc[VSTOP]  =	19;

  if(!tcsetattr(fd, TCSANOW, &(__P_tty_buf)))
    {
    P_error_set_errno(P_ETCATTR);
    return(FALSE);
    }

  return(TRUE);
  }

/* end of source file */
