/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: vector.c,v $
   Revision 1.2  2000/07/04 12:54:40  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:49  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* Local headers */

#include "pingutil.h"

/* Functions */

p_vector_t *P_vector_start(uint_t resize_rate)
  {
  p_vector_t *v = P_new(p_vector_t);

  v->len = v->c = 0;
  v->vs = v->v = P_newa(resize_rate, char *);
  v->blk = 1;
  v->blksz = resize_rate;

  return(v);
  }

/*
 */

bool_t P_vector_store(p_vector_t *v, const char *s)
  {
 
  if(!v || !s)
    return(FALSE);

  *(v->v) = (char *)s;
  (v->len)++;
  if(++(v->c) == v->blksz)
    {
    v->c = 0;
    v->vs = P_realloc(v->vs, v->blksz * ++(v->blk), char *);
    v->v = v->vs + ((v->blk - 1) * v->blksz);
    }
  else
    (v->v)++;

  return(TRUE);
  }

/*
 */

char **P_vector_end(p_vector_t *v, uint_t *len)
  {
  char **r;

  if(!v) return(NULL);

  if(len) *len = v->len;
  if(!v->len)
    {
    P_free(v->vs);
    P_free(v);
    return(NULL);
    }

  r = P_realloc(v->vs, ++(v->len), char *);
  *(r + v->len - 1) = NULL;
  P_free(v);

  return(r);
  }

/*
 */

void P_vector_abort(p_vector_t *v)
  {

  if(v)
    {
    P_free_vec(v->vs);
    P_free(v);
    }
  }

/* end of source file */
